@@ -17,29 +17,30 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#ifndef __cplusplus
-#    include <stdbool.h>
-#endif
+#include "atom_sink.h"
+#include "peaks.h"
+#include "uris.h"
+
+#include "lv2/atom/atom.h"
+#include "lv2/atom/forge.h"
+#include "lv2/atom/util.h"
+#include "lv2/core/lv2.h"
+#include "lv2/core/lv2_util.h"
+#include "lv2/log/log.h"
+#include "lv2/log/logger.h"
+#include "lv2/midi/midi.h"
+#include "lv2/state/state.h"
+#include "lv2/urid/urid.h"
+#include "lv2/worker/worker.h"
 
 #include <sndfile.h>
 
-#include "lv2/lv2plug.in/ns/ext/atom/forge.h"
-#include "lv2/lv2plug.in/ns/ext/atom/util.h"
-#include "lv2/lv2plug.in/ns/ext/log/log.h"
-#include "lv2/lv2plug.in/ns/ext/log/logger.h"
-#include "lv2/lv2plug.in/ns/ext/midi/midi.h"
-#include "lv2/lv2plug.in/ns/ext/patch/patch.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/urid/urid.h"
-#include "lv2/lv2plug.in/ns/ext/worker/worker.h"
-#include "lv2/lv2plug.in/ns/lv2core/lv2.h"
-#include "lv2/lv2plug.in/ns/lv2core/lv2_util.h"
-
-#include "uris.h"
-#include "atom_sink.h"
+#include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 enum {
 	SAMPLER_CONTROL = 0,
@@ -60,16 +61,15 @@ typedef struct {
 	LV2_Worker_Schedule* schedule;
 	LV2_Log_Logger       logger;
 
-	// Forge for creating atoms
-	LV2_Atom_Forge forge;
-
 	// Ports
 	const LV2_Atom_Sequence* control_port;
 	LV2_Atom_Sequence*       notify_port;
 	float*                   output_port;
 
-	// Forge frame for notify port (for writing worker replies)
-	LV2_Atom_Forge_Frame notify_frame;
+	// Communication utilities
+	LV2_Atom_Forge_Frame notify_frame;  ///< Cached for worker replies
+	LV2_Atom_Forge       forge;         ///< Forge for writing atoms in run thread
+	PeaksSender          psend;         ///< Audio peaks sender
 
 	// URIs
 	SamplerURIs uris;
@@ -110,22 +110,28 @@ load_sample(LV2_Log_Logger* logger, const char* path)
 	lv2_log_trace(logger, "Loading %s\n", path);
 
 	const size_t   path_len = strlen(path);
-	Sample* const  sample   = (Sample*)malloc(sizeof(Sample));
+	Sample* const  sample   = (Sample*)calloc(1, sizeof(Sample));
 	SF_INFO* const info     = &sample->info;
 	SNDFILE* const sndfile  = sf_open(path, SFM_READ, info);
-
-	if (!sndfile || !info->frames || (info->channels != 1)) {
-		lv2_log_error(logger, "Failed to open sample '%s'\n", path);
-		free(sample);
-		return NULL;
-	}
-
-	// Read data
-	float* const data = (float*)malloc(sizeof(float) * info->frames);
-	if (!data) {
+	float*         data     = NULL;
+	bool           error    = true;
+	if (!sndfile || !info->frames) {
+		lv2_log_error(logger, "Failed to open %s\n", path);
+	} else if (info->channels != 1) {
+		lv2_log_error(logger, "%s has %d channels\n", path, info->channels);
+	} else if (!(data = (float*)malloc(sizeof(float) * info->frames))) {
 		lv2_log_error(logger, "Failed to allocate memory for sample\n");
+	} else {
+		error = false;
+	}
+
+	if (error) {
+		free(sample);
+		free(data);
+		sf_close(sndfile);
 		return NULL;
 	}
+
 	sf_seek(sndfile, 0ul, SEEK_SET);
 	sf_read_float(sndfile, data, info->frames);
 	sf_close(sndfile);
@@ -155,7 +161,7 @@ free_sample(Sampler* self, Sample* sample)
 
    This is called for every piece of work scheduled in the audio thread using
    self->schedule->schedule_work().  A reply can be sent back to the audio
-   thread using the provided respond function.
+   thread using the provided `respond` function.
 */
 static LV2_Worker_Status
 work(LV2_Handle                  instance,
@@ -214,13 +220,11 @@ work_response(LV2_Handle  instance,
 	                      old_sample };
 	self->schedule->schedule_work(self->schedule->handle, sizeof(msg), &msg);
 
-	if (strcmp(old_sample->path, new_sample->path)) {
-		// Send a notification that we're using a new sample
-		lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
-		write_set_file(&self->forge, &self->uris,
-		               new_sample->path,
-		               new_sample->path_len);
-	}
+	// Send a notification that we're using a new sample
+	lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
+	write_set_file(&self->forge, &self->uris,
+		       new_sample->path,
+		       new_sample->path_len);
 
 	return LV2_WORKER_SUCCESS;
 }
@@ -275,6 +279,9 @@ instantiate(const LV2_Descriptor*     descriptor,
 	// Map URIs and initialise forge
 	map_sampler_uris(self->map, &self->uris);
 	lv2_atom_forge_init(&self->forge, self->map);
+	peaks_sender_init(&self->psend, self->map);
+
+	self->gain = 1.0;
 
 	return (LV2_Handle)self;
 }
@@ -302,15 +309,119 @@ deactivate(LV2_Handle instance)
 /** Define a macro for converting a gain in dB to a coefficient. */
 #define DB_CO(g) ((g) > -90.0f ? powf(10.0f, (g) * 0.05f) : 0.0f)
 
+/**
+   Handle an incoming event in the audio thread.
+
+   This performs any actions triggered by an event, such as the start of sample
+   playback, a sample change, or responding to requests from the UI.
+*/
 static void
-run(LV2_Handle instance,
-    uint32_t   sample_count)
+handle_event(Sampler* self, LV2_Atom_Event* ev)
 {
-	Sampler*     self        = (Sampler*)instance;
-	SamplerURIs* uris        = &self->uris;
-	sf_count_t   start_frame = 0;
-	sf_count_t   pos         = 0;
-	float*       output      = self->output_port;
+	SamplerURIs* uris       = &self->uris;
+	PeaksURIs*   peaks_uris = &self->psend.uris;
+
+	if (ev->body.type == uris->midi_Event) {
+		const uint8_t* const msg = (const uint8_t*)LV2_ATOM_BODY_CONST(ev);
+		switch (lv2_midi_message_type(msg)) {
+		case LV2_MIDI_MSG_NOTE_ON:
+			self->frame = 0;
+			self->play  = true;
+			break;
+		default:
+			break;
+		}
+	} else if (lv2_atom_forge_is_object_type(&self->forge, ev->body.type)) {
+		const LV2_Atom_Object* obj = (const LV2_Atom_Object*)&ev->body;
+		if (obj->body.otype == uris->patch_Set) {
+			// Get the property and value of the set message
+			const LV2_Atom* property = NULL;
+			const LV2_Atom* value    = NULL;
+			lv2_atom_object_get(obj,
+			                    uris->patch_property, &property,
+			                    uris->patch_value,    &value,
+			                    0);
+			if (!property) {
+				lv2_log_error(&self->logger, "Set message with no property\n");
+				return;
+			} else if (property->type != uris->atom_URID) {
+				lv2_log_error(&self->logger, "Set property is not a URID\n");
+				return;
+			}
+
+			const uint32_t key = ((const LV2_Atom_URID*)property)->body;
+			if (key == uris->eg_sample) {
+				// Sample change, send it to the worker.
+				lv2_log_trace(&self->logger, "Scheduling sample change\n");
+				self->schedule->schedule_work(self->schedule->handle,
+				                              lv2_atom_total_size(&ev->body),
+				                              &ev->body);
+			} else if (key == uris->param_gain) {
+				// Gain change
+				if (value->type == uris->atom_Float) {
+					self->gain = DB_CO(((LV2_Atom_Float*)value)->body);
+				}
+			}
+		} else if (obj->body.otype == uris->patch_Get && self->sample) {
+			const LV2_Atom_URID* accept  = NULL;
+			const LV2_Atom_Int*  n_peaks = NULL;
+			lv2_atom_object_get_typed(
+				obj,
+				uris->patch_accept,      &accept,  uris->atom_URID,
+				peaks_uris->peaks_total, &n_peaks, peaks_uris->atom_Int, 0);
+			if (accept && accept->body == peaks_uris->peaks_PeakUpdate) {
+				// Received a request for peaks, prepare for transmission
+				peaks_sender_start(&self->psend,
+				                   self->sample->data,
+				                   self->sample->info.frames,
+				                   n_peaks->body);
+			} else {
+				// Received a get message, emit our state (probably to UI)
+				lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
+				write_set_file(&self->forge, &self->uris,
+				               self->sample->path,
+				               self->sample->path_len);
+			}
+		} else {
+			lv2_log_trace(&self->logger,
+			              "Unknown object type %d\n", obj->body.otype);
+		}
+	} else {
+		lv2_log_trace(&self->logger,
+		              "Unknown event type %d\n", ev->body.type);
+	}
+
+}
+
+/**
+   Output audio for a slice of the current cycle.
+*/
+static void
+render(Sampler* self, uint32_t start, uint32_t end)
+{
+	float* output = self->output_port;
+
+	if (self->play && self->sample) {
+		// Start/continue writing sample to output
+		for (; start < end; ++start) {
+			output[start] = self->sample->data[self->frame] * self->gain;
+			if (++self->frame == self->sample->info.frames) {
+				self->play = false;  // Reached end of sample
+				break;
+			}
+		}
+	}
+
+	// Write silence to remaining buffer
+	for (; start < end; ++start) {
+		output[start] = 0.0f;
+	}
+}
+
+static void
+run(LV2_Handle instance, uint32_t sample_count)
+{
+	Sampler* self = (Sampler*)instance;
 
 	// Set up forge to write directly to notify output port.
 	const uint32_t notify_capacity = self->notify_port->atom.size;
@@ -330,93 +441,28 @@ run(LV2_Handle instance,
 		self->sample_changed = false;
 	}
 
-	// Read incoming events
+	// Iterate over incoming events, emitting audio along the way
+	self->frame_offset = 0;
 	LV2_ATOM_SEQUENCE_FOREACH(self->control_port, ev) {
+		// Render output up to the time of this event
+		render(self, self->frame_offset, ev->time.frames);
+
+		/* Update current frame offset to this event's time.  This is stored in
+		   the instance because it is used for sychronous worker event
+		   execution.  This allows a sample load event to be executed with
+		   sample accuracy when running in a non-realtime context (such as
+		   exporting a session). */
 		self->frame_offset = ev->time.frames;
-		if (ev->body.type == uris->midi_Event) {
-			const uint8_t* const msg = (const uint8_t*)(ev + 1);
-			switch (lv2_midi_message_type(msg)) {
-			case LV2_MIDI_MSG_NOTE_ON:
-				start_frame = ev->time.frames;
-				self->frame = 0;
-				self->play  = true;
-				break;
-			default:
-				break;
-			}
-		} else if (lv2_atom_forge_is_object_type(&self->forge, ev->body.type)) {
-			const LV2_Atom_Object* obj = (const LV2_Atom_Object*)&ev->body;
-			if (obj->body.otype == uris->patch_Set) {
-				// Get the property and value of the set message
-				const LV2_Atom* property = NULL;
-				const LV2_Atom* value    = NULL;
-				lv2_atom_object_get(obj,
-				                    uris->patch_property, &property,
-				                    uris->patch_value,    &value,
-				                    0);
-				if (!property) {
-					lv2_log_error(&self->logger,
-					              "patch:Set message with no property\n");
-					continue;
-				} else if (property->type != uris->atom_URID) {
-					lv2_log_error(&self->logger,
-					              "patch:Set property is not a URID\n");
-					continue;
-				}
 
-				const uint32_t key = ((const LV2_Atom_URID*)property)->body;
-				if (key == uris->eg_sample) {
-					// Sample change, send it to the worker.
-					lv2_log_trace(&self->logger, "Scheduling sample change\n");
-					self->schedule->schedule_work(self->schedule->handle,
-					                              lv2_atom_total_size(&ev->body),
-					                              &ev->body);
-				} else if (key == uris->param_gain) {
-					// Gain change
-					if (value->type == uris->atom_Float) {
-						self->gain = DB_CO(((LV2_Atom_Float*)value)->body);
-					}
-				}
-			} else if (obj->body.otype == uris->patch_Get) {
-				// Received a get message, emit our state (probably to UI)
-				lv2_atom_forge_frame_time(&self->forge, self->frame_offset);
-				write_set_file(&self->forge, &self->uris,
-				               self->sample->path,
-				               self->sample->path_len);
-			} else {
-				lv2_log_trace(&self->logger,
-				              "Unknown object type %d\n", obj->body.otype);
-			}
-		} else {
-			lv2_log_trace(&self->logger,
-			              "Unknown event type %d\n", ev->body.type);
-		}
+		// Process this event
+		handle_event(self, ev);
 	}
 
-	// Render the sample (possibly already in progress)
-	if (self->play) {
-		uint32_t       f  = self->frame;
-		const uint32_t lf = self->sample->info.frames;
+	// Use available space after any emitted events to send peaks
+	peaks_sender_send(&self->psend, &self->forge, sample_count, self->frame_offset);
 
-		for (pos = 0; pos < start_frame; ++pos) {
-			output[pos] = 0;
-		}
-
-		for (; pos < sample_count && f < lf; ++pos, ++f) {
-			output[pos] = self->sample->data[f] * self->gain;
-		}
-
-		self->frame = f;
-
-		if (f == lf) {
-			self->play = false;
-		}
-	}
-
-	// Add zeros to end if sample not long enough (or not playing)
-	for (; pos < sample_count; ++pos) {
-		output[pos] = 0.0f;
-	}
+	// Render output for the rest of the cycle past the last event
+	render(self, self->frame_offset, sample_count);
 }
 
 static LV2_State_Status
@@ -496,9 +542,12 @@ restore(LV2_Handle                  instance,
 	if (!self->activated || !schedule) {
 		// No scheduling available, load sample immediately
 		lv2_log_trace(&self->logger, "Synchronous restore\n");
-		free_sample(self, self->sample);
-		self->sample         = load_sample(&self->logger, path);
-		self->sample_changed = true;
+		Sample* sample = load_sample(&self->logger, path);
+		if (sample) {
+			free_sample(self, self->sample);
+			self->sample         = sample;
+			self->sample_changed = true;
+		}
 	} else {
 		// Schedule sample to be loaded by the provided worker
 		lv2_log_trace(&self->logger, "Scheduling restore\n");
@@ -513,6 +562,8 @@ restore(LV2_Handle                  instance,
 		free(buf);
 	}
 
+	free(path);
+
 	return LV2_STATE_SUCCESS;
 }
 
