@@ -18,15 +18,18 @@
 #ifndef SAMPLER_URIS_H
 #define SAMPLER_URIS_H
 
-#include "lv2/lv2plug.in/ns/ext/log/log.h"
-#include "lv2/lv2plug.in/ns/ext/midi/midi.h"
-#include "lv2/lv2plug.in/ns/ext/state/state.h"
-#include "lv2/lv2plug.in/ns/ext/parameters/parameters.h"
+#include "lv2/log/log.h"
+#include "lv2/midi/midi.h"
+#include "lv2/parameters/parameters.h"
+#include "lv2/patch/patch.h"
+#include "lv2/state/state.h"
+
+#include <stdio.h>
 
 #define EG_SAMPLER_URI          "http://lv2plug.in/plugins/eg-sampler"
-#define EG_SAMPLER__sample      EG_SAMPLER_URI "#sample"
 #define EG_SAMPLER__applySample EG_SAMPLER_URI "#applySample"
 #define EG_SAMPLER__freeSample  EG_SAMPLER_URI "#freeSample"
+#define EG_SAMPLER__sample      EG_SAMPLER_URI "#sample"
 
 typedef struct {
 	LV2_URID atom_Float;
@@ -36,12 +39,13 @@ typedef struct {
 	LV2_URID atom_URID;
 	LV2_URID atom_eventTransfer;
 	LV2_URID eg_applySample;
-	LV2_URID eg_sample;
 	LV2_URID eg_freeSample;
+	LV2_URID eg_sample;
 	LV2_URID midi_Event;
 	LV2_URID param_gain;
 	LV2_URID patch_Get;
 	LV2_URID patch_Set;
+	LV2_URID patch_accept;
 	LV2_URID patch_property;
 	LV2_URID patch_value;
 } SamplerURIs;
@@ -62,17 +66,21 @@ map_sampler_uris(LV2_URID_Map* map, SamplerURIs* uris)
 	uris->param_gain         = map->map(map->handle, LV2_PARAMETERS__gain);
 	uris->patch_Get          = map->map(map->handle, LV2_PATCH__Get);
 	uris->patch_Set          = map->map(map->handle, LV2_PATCH__Set);
+	uris->patch_accept       = map->map(map->handle, LV2_PATCH__accept);
 	uris->patch_property     = map->map(map->handle, LV2_PATCH__property);
 	uris->patch_value        = map->map(map->handle, LV2_PATCH__value);
 }
 
 /**
- * Write a message like the following to @p forge:
- * []
- *     a patch:Set ;
- *     patch:property eg:sample ;
- *     patch:value </home/me/foo.wav> .
- */
+   Write a message like the following to `forge`:
+   [source,turtle]
+   ----
+   []
+   a patch:Set ;
+   patch:property eg:sample ;
+   patch:value </home/me/foo.wav> .
+   ----
+*/
 static inline LV2_Atom_Forge_Ref
 write_set_file(LV2_Atom_Forge*    forge,
                const SamplerURIs* uris,
@@ -93,12 +101,15 @@ write_set_file(LV2_Atom_Forge*    forge,
 }
 
 /**
- * Get the file path from a message like:
- * []
- *     a patch:Set ;
- *     patch:property eg:sample ;
- *     patch:value </home/me/foo.wav> .
- */
+   Get the file path from `obj` which is a message like:
+   [source,turtle]
+   ----
+   []
+   a patch:Set ;
+   patch:property eg:sample ;
+   patch:value </home/me/foo.wav> .
+   ----
+*/
 static inline const char*
 read_set_file(const SamplerURIs*     uris,
               const LV2_Atom_Object* obj)
